import React, { useState, useEffect } from "react";
import {
    ControlLabel,
    FormGroup,
    Glyphicon,
    Tooltip,
    HelpBlock
} from "react-bootstrap";
import CodeMirror from '../../../../libs/codemirror/react-codemirror-suspense';
import isEqual from "lodash/isEqual";
import { cswGetRecordsXml } from "../../../../api/CSW";
import OverlayTrigger from "../../../misc/OverlayTrigger";
import Message from "../../../I18N/Message";

const options = {
    mode: "xml",
    theme: "material",
    lineNumbers: true,
    lineWrapping: true,
    autoRefresh: true,
    indentUnit: 2,
    tabSize: 2
};
const FilterInfo = ({ glyph = "info-sign", className = "", tooltip }) => (
    <OverlayTrigger placement={"bottom"} overlay={tooltip}>
        <Glyphicon
            style={{ marginLeft: 4 }}
            glyph={glyph}
            className={className}
        />
    </OverlayTrigger>
);

const tooltip = _type => {
    let msgId = `catalog.filter.${_type}.info`;
    if (_type === "error") msgId = `catalog.filter.error`;
    return (
        <Tooltip id={"filter"}>
            <Message msgId={msgId} />
        </Tooltip>
    );
};
const renderError = (
    <FilterInfo
        tooltip={tooltip("error")}
        glyph={"exclamation-mark"}
        className={"text-danger"}
    />
);
const renderHelpText = (
    <HelpBlock style={{ fontSize: 12 }}>
        <Message msgId={`catalog.filter.dynamic.helpText`} />
    </HelpBlock>
);

const FilterCode = ({ type, code, setCode, error }) => {
    const filterProp = `${type}Filter`;
    return (
        <FormGroup>
            <div className="label-pre-textarea">
                <ControlLabel>
                    <Message msgId={`catalog.filter.${type}.label`} />
                </ControlLabel>
                <FilterInfo tooltip={tooltip(type)} />
                {error[type] && renderError}
            </div>
            <div className="textarea-code">
                <CodeMirror
                    value={code[filterProp]}
                    options={options}
                    onBeforeChange={(_, __, value) => {
                        setCode({ ...code, [filterProp]: value });
                    }}
                />
                {type === 'dynamic' && renderHelpText}
            </div>
        </FormGroup>
    );
};

export default ({
    onChangeServiceProperty = () => {},
    filter: { staticFilter, dynamicFilter } = {}
} = {}) => {
    const [error, setError] = useState({});
    const [code, setCode] = useState({ staticFilter, dynamicFilter });

    const cmProps = { code, setCode, error };
    const isValid = value => {
        const _filter = cswGetRecordsXml({
            filterXml: value,
            startPosition: 1,
            maxRecords: 4,
            sortBy: ''
        });
        return !new DOMParser()
            .parseFromString(_filter, "application/xml")
            ?.getElementsByTagName("parsererror")?.length;
    };

    useEffect(() => {
        const validFt = isValid(code.staticFilter);
        validFt &&
            !isEqual(code.staticFilter, staticFilter) &&
            onChangeServiceProperty("filter.staticFilter", code.staticFilter);
        setError({ ...error, "static": !validFt });
    }, [code.staticFilter]);

    useEffect(() => {
        const validFt = isValid(code.dynamicFilter);
        validFt &&
            !isEqual(code.dynamicFilter, dynamicFilter) &&
            onChangeServiceProperty("filter.dynamicFilter", code.dynamicFilter);
        setError({ ...error, dynamic: !validFt });
    }, [code.dynamicFilter]);

    return (
        <div className={"catalog-csw-filters"} style={{ position: 'relative', zIndex: 0 }}>
            <FilterCode type={"static"} {...cmProps} />
            <FilterCode type={"dynamic"} {...cmProps} />
        </div>
    );
};
