import React from 'react';
import expect from 'expect';
import ReactDOM from 'react-dom';

import {
    CLASSIFICATIONS,
    RANGE_CLASSIFICATIONS,
    DATASET_1,
    DATASET_2,
    DATASET_3,
    DATASET_5_UNORDERED,
    DATASET_5_ORDERED,
    DATASET_4,
    LABELLED_CLASSIFICATION,
    LABELLED_RANGE_CLASSIFICATION,
    UNLABELLED_CLASSIFICATION,
    UNLABELLED_CLASSIFICATION_3,
    UNLABELLED_CLASSIFICATION_5_ORDERED,
    UNLABELLED_RANGE_CLASSIFICATION,
    PIE_CHART_TEMPLATE_LABELS_CLASSIFICATION,
    PIE_CHART_TEMPLATE_LABELS_RANGE_CLASSIFICATION,
    SPLIT_DATASET_2,
    SPLIT_DATASET_3,
    SPLIT_DATASET_5_ORDERED,
    TEMPLATE_LABELS_CLASSIFICATION,
    TEMPLATE_LABELS_RANGE_CLASSIFICATION,
    DATASET_WITH_DATES,
    SPLIT_DATASET_4
} from './sample_data';
import WidgetChart, { toPlotly } from '../WidgetChart';
import { createClassifyGeoJSONSync } from '../../../api/GeoJSONClassification';

import * as simpleStatistics from 'simple-statistics';
const classifyGeoJSONSync = createClassifyGeoJSONSync(simpleStatistics);

describe('WidgetChart', () => {
    beforeEach((done) => {
        document.body.innerHTML = '<div id="container"></div>';
        setTimeout(done);
    });
    afterEach((done) => {
        ReactDOM.unmountComponentAtNode(document.getElementById("container"));
        document.body.innerHTML = '';
        setTimeout(done);
    });
    it('rendering pie', (done) => {
        const check = ({ data, layout }, graphDiv) => {
            try {
                expect(graphDiv).toExist();
                expect(layout.showlegend).toBeFalsy();
                expect(layout.autosize).toBeFalsy();
                expect(layout.automargin).toBeFalsy();
                expect(layout.legend).toBeFalsy();
                expect(data.length).toEqual(1);
                // use yAxis dataKey as default label
                expect(data[0].name).toEqual(DATASET_1.series[0].dataKey);
                // original values
                expect(DATASET_1.data.map(d => d[DATASET_1.series[0].dataKey])).toEqual([0, 1, 2, 3]);
                // data values mapped and sorted
                expect(data[0].values).toEqual([ 3, 2, 1, 0 ]);
                // original labels
                expect(DATASET_1.data.map(d => d[DATASET_1.xAxis.dataKey])).toEqual(['Page A', 'Page B', 'Page C', 'Page D']);
                // data labels mapped and sorted
                expect(data[0].labels).toEqual(['Page D', 'Page C', 'Page B', 'Page A']);
            } catch (e) {
                done(e);
            }
            done();
        };
        ReactDOM.render(<WidgetChart onInitialized={check} {...DATASET_1} type="pie" />, document.getElementById("container"));
    });
    it('rendering line', (done) => {
        const check = ({ data, layout }, graphDiv) => {
            expect(graphDiv).toExist();
            expect(layout.showlegend).toBeFalsy();
            expect(layout.autosize).toBeFalsy();
            expect(layout.automargin).toBeFalsy();
            expect(layout.hovermode).toBe('x unified');
            expect(data.length).toEqual(1);
            // use yAxis dataKey as default label
            expect(data[0].name).toEqual(DATASET_1.series[0].dataKey);
            // data values mapped
            data[0].y.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.series[0].dataKey]));
            // data labels mapped
            data[0].x.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.xAxis.dataKey]));
            done();
        };
        ReactDOM.render(<WidgetChart onInitialized={check} {...DATASET_1} type="line" />, document.getElementById("container"));
    });
    it('rendering bar', (done) => {
        const check = ({ data, layout }, graphDiv) => {
            expect(graphDiv).toExist();
            expect(layout.showlegend).toBeFalsy();
            expect(layout.autosize).toBeFalsy();
            expect(layout.automargin).toBeFalsy();
            expect(layout.hovermode).toBe('x unified');
            expect(data.length).toEqual(1);
            // use yAxis dataKey as default label
            expect(data[0].name).toEqual(DATASET_1.series[0].dataKey);
            // data values mapped
            data[0].y.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.series[0].dataKey]));
            // data labels mapped
            data[0].x.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.xAxis.dataKey]));
            done();
        };
        ReactDOM.render(<WidgetChart onInitialized={check} {...DATASET_1} type="bar" />, document.getElementById("container"));
    });
    it('rendering multi traces', (done) => {
        const check = ({ data, layout }, graphDiv) => {
            try {
                expect(graphDiv).toBeTruthy();
                expect(data.length).toBe(2);
                expect(data).toEqual([
                    {
                        mode: 'lines',
                        name: 'population',
                        hovertemplate: '%{y:d}<extra></extra>',
                        x: [ 'A', 'B', 'C' ],
                        y: [ 500, 1000, 600 ],
                        line: {
                            color: '#ff0000',
                            width: 2
                        }
                    },
                    {
                        mode: 'lines',
                        name: 'cars',
                        hovertemplate: '%{y:d}<extra></extra>',
                        x: [ 'A', 'B', 'C' ],
                        y: [ 400, 600, 300 ],
                        yaxis: 'y2',
                        line: {
                            color: '#0000ff',
                            width: 2
                        }
                    }
                ]);
                expect(layout.yaxis).toBeTruthy();
                expect(layout.yaxis2).toBeTruthy();
                expect(layout.yaxis2.side).toBe('right');
                expect(layout.xaxis).toBeTruthy();
            } catch (e) {
                done(e);
            }
            done();
        };
        ReactDOM.render(<WidgetChart
            onInitialized={check}
            yAxisOpts={[
                { id: 0 },
                { id: 'y-axis-1', side: 'right' }
            ]}
            data={[
                [
                    { population: 500, label: 'A' },
                    { population: 1000, label: 'B' },
                    { population: 600, label: 'C' }
                ],
                [
                    { cars: 400, label: 'A' },
                    { cars: 600, label: 'B' },
                    { cars: 300, label: 'C' }
                ]
            ]}
            traces={[
                {
                    type: 'line',
                    options: {
                        groupByAttributes: 'label',
                        aggregationAttribute: 'population'
                    },
                    style: {
                        line: {
                            color: '#ff0000',
                            width: 2
                        }
                    }
                },
                {
                    type: 'line',
                    yaxis: 'y-axis-1',
                    options: {
                        groupByAttributes: 'label',
                        aggregationAttribute: 'cars'
                    },
                    style: {
                        line: {
                            color: '#0000ff',
                            width: 2
                        }
                    }
                }
            ]}
        />, document.getElementById("container"));
    });
});

const TYPES = ['pie', 'line', 'bar'];

describe('Widget Chart: data conversions ', () => {
    describe('common options', () => {
        function testAllTypes(props, handler) {
            TYPES.map(type => toPlotly({
                type,
                ...props,
                classifyGeoJSONSync
            })).map(handler);
        }
        it('defaults', () => {
            testAllTypes(DATASET_1, ({data, config, layout}) => {
                // default settings valid for all chart types
                expect(layout.showlegend).toBeFalsy();
                expect(layout.autosize).toBeFalsy();
                expect(layout.automargin).toBeFalsy();
                expect(data.length).toEqual(1);
                // use yAxis dataKey as default label
                expect(data[0].name).toEqual(DATASET_1.series[0].dataKey);
                // hide logo
                expect(config.displaylogo).toBeFalsy();
            });
        });
        const THRESHOLD = 350;
        it('test dimension are passed', () => {
            testAllTypes({ ...DATASET_1, width: 200, height: 200 }, ({ layout }) => {
                expect(layout.width).toBe(200);
                expect(layout.height).toBe(200);
            });
        });
        it(`show toolbar only if witdth > ${THRESHOLD}, with correct margin`, () => {
            testAllTypes({ ...DATASET_1, width: THRESHOLD }, ({ config, layout }) => {
                expect(config.displayModeBar).toEqual(false);
                expect(layout.margin.t).toEqual(8);
            });
            testAllTypes({ ...DATASET_1, width: THRESHOLD + 1 }, ({ config, layout }) => {
                expect(config.displayModeBar).toEqual(true);
                expect(layout.margin.t).toEqual(25);
            });
        });
        it('show legend', () => {
            testAllTypes({ ...DATASET_1, legend: true }, ({ layout }) => {
                expect(layout.showlegend).toBeTruthy();
            });
        });
    });
    describe('Pie chart', () => {
        it('basic chart options', () => {
            const { data } = toPlotly({
                type: 'pie',
                ...DATASET_1,
                classifyGeoJSONSync
            });
            // DATA
            expect(data.length).toBe(1);
            expect(data[0].type).toBe('pie');
            // this avoids text to overflow the chart div when rendered outside the widget
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_1.data.map(d => d[DATASET_1.series[0].dataKey])).toEqual([ 0, 1, 2, 3 ]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 3, 2, 1, 0 ]);
            // original labels
            expect(DATASET_1.data.map(d => d[DATASET_1.xAxis.dataKey])).toEqual(['Page A', 'Page B', 'Page C', 'Page D']);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['Page D', 'Page C', 'Page B', 'Page A']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual(['#08306b', '#3787c0', '#abd0e6', '#f7fbff']);
        });
        it('Pie chart with modified options', () => {
            const { data, layout } = toPlotly({
                type: 'pie',
                width: 500,
                ...DATASET_1,
                classifyGeoJSONSync
            });
            // DATA
            expect(data.length).toBe(1);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_1.data.map(d => d[DATASET_1.series[0].dataKey])).toEqual([ 0, 1, 2, 3 ]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 3, 2, 1, 0 ]);
            // original labels
            expect(DATASET_1.data.map(d => d[DATASET_1.xAxis.dataKey])).toEqual(['Page A', 'Page B', 'Page C', 'Page D']);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['Page D', 'Page C', 'Page B', 'Page A']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual(['#08306b', '#3787c0', '#abd0e6', '#f7fbff']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 25, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // Modified margin based on width
            expect(layout.legend).toBeTruthy();
            expect(layout.legend).toEqual({x: 1.05, y: 0.5}); // Legend option to right and centered based on width
        });

        it('custom colors', () => {
            const { data } = toPlotly({
                traces: [{
                    type: 'pie',
                    style: {
                        ramp: 'spectral',
                        method: 'uniqueInterval'
                    },
                    options: {
                        groupByAttributes: DATASET_1.xAxis.dataKey,
                        aggregationAttribute: DATASET_1.series[0].dataKey
                    }
                }],
                classifyGeoJSONSync,
                data: [DATASET_1.data]
            });
            expect(data[0].marker.colors).toEqual([ '#fee825', '#39ad7a', '#365d8d', '#440154' ]);
        });

        it('should add percentage to label with includeLegendPercent', () => {
            const { data } = toPlotly({
                traces: [{
                    type: 'pie',
                    includeLegendPercent: true,
                    options: {
                        groupByAttributes: DATASET_1.xAxis.dataKey,
                        aggregationAttribute: DATASET_1.series[0].dataKey
                    }
                }],
                classifyGeoJSONSync,
                data: [DATASET_1.data]
            });
            expect(data[0].labels).toEqual([ 'Page D - 50.0%', 'Page C - 33.3%', 'Page B - 16.7%', 'Page A - 0.00%' ]);
        });
    });
    it('color mapping classification is undefined - Pie Chart', () => {
        const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", name: 'global.colors.custom' };
        const { data, layout } = toPlotly({
            type: 'pie',
            autoColorOptions,
            options: {
                classificationAttributeType: 'string'
            },
            classifyGeoJSONSync,
            ...DATASET_2
        });
        expect(data.length).toBe(1);
        expect(data[0].type).toBe('pie');
        expect(data[0].textposition).toEqual('inside');
        // original values
        expect(DATASET_2.data.map(d => d[DATASET_2.series[0].dataKey])).toEqual([0, 1]);
        // data values mapped and sorted
        expect(data[0].values).toEqual([ 1, 0 ]);
        // original labels
        expect(DATASET_2.data.map(d => d[DATASET_2.xAxis.dataKey])).toEqual(['Page A', 'Page B']);
        // data labels mapped and sorted
        expect(data[0].labels).toEqual(['Default', 'Default']);
        // colors are those defined by the user
        expect(data[0].marker.colors).toEqual(['#00ff00', '#00ff00']);
        // LAYOUT
        expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
    });
    describe('1) Pie chart - Color coded custom classifications with absolute values', () => {
        it('custom classified colors - using custom labels and colors only 1', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", classification: LABELLED_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(2);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_2.data.map(d => d[DATASET_2.series[0].dataKey])).toEqual([0, 1]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 0, 1 ]);
            // original labels
            expect(DATASET_2.data.map(d => d[DATASET_2.xAxis.dataKey])).toEqual(['Page A', 'Page B']);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual([ '1) Class 1', '2) Class 2' ]);
            expect(data[1].labels).toEqual([ 'Page A (1)', 'Page B (2)' ]);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual(['#ff0000', '#0000ff']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
        it('custom classified colors - using default labels and colors', () => {
            const autoColorOptions = {
                defaultCustomColor: "#00ff00",
                defaultClassLabel: "Default label",
                classification: UNLABELLED_CLASSIFICATION,
                name: 'global.colors.custom'
            };

            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_3
            });
            expect(data.length).toBe(2);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_3.data.map(d => d[DATASET_3.series[0].dataKey])).toEqual([0, 1, 4, 5]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 0, 1, 5, 4 ]);
            // original labels
            expect(DATASET_3.data.map(d => d[DATASET_3.xAxis.dataKey])).toEqual(['Page A', 'Page B', 'Page E', 'Page F']);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['1) class1', '2) class2', '3) Default label', '3) Default label']);
            expect(data[1].labels).toEqual(['Page A (1)', 'Page B (2)', 'Page F (3)', 'Page E (3)']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual(['#ff0000', '#0000ff', '#00ff00', '#00ff00']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
        it('custom classified colors - using default labels and colors, wrong order', () => {
            const classification = UNLABELLED_CLASSIFICATION_5_ORDERED;
            const autoColorOptions = {
                defaultCustomColor: "#00ff00",
                defaultClassLabel: "Default",
                classification,
                name: 'global.colors.custom'
            };
            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_5_UNORDERED
            });
            expect(data.length).toBe(2);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_5_ORDERED.data.map(d => d[DATASET_5_ORDERED.series[0].dataKey])).toEqual([1, 0, 1, 10, 1, 100]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 1, 0, 10, 1, 100, 1 ]);
            // original labels
            expect(DATASET_5_ORDERED.data.map(d => d[CLASSIFICATIONS.dataKey])).toEqual([ '1', '1', '2', '2', '3', '3' ]);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['1) 1', '1) 1', '2) 2', '2) 2', '3) 3', '3) 3']);
            expect(data[1].labels).toEqual(['Page A (1)', 'Page B (1)', 'Page B (2)', 'Page A (2)', 'Page B (3)', 'Page A (3)']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual([ '#0000ff', '#0000ff', '#00FF00', '#00FF00', '#ff0000', '#ff0000']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
        it('custom classified colors - using templatized labels and custom colors only - pie charts', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "", classification: PIE_CHART_TEMPLATE_LABELS_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(1);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_2.data.map(d => d[DATASET_2.series[0].dataKey])).toEqual([0, 1]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 0, 1 ]);
            // original labels
            expect(DATASET_2.data.map(d => d[CLASSIFICATIONS.dataKey])).toEqual([ 'class1', 'class2' ]);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['Page A - Class 1', 'Page B - Class 2']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual([ '#ff0000', '#0000ff']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
        it('custom color ramp', () => {
            const autoColorOptions = {
                defaultCustomColor: "#00FF00",
                defaultClassLabel: "Default",
                name: 'global.colors.custom',
                base: 190,
                range: 0,
                s: 0.95,
                v: 0.63
            };
            const { data } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(1);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            data[0].marker.colors.map((v) => {
                expect(v).toBe(autoColorOptions.defaultCustomColor);
            });
        });
    });
    describe('2) Pie chart - Color coded custom classifications with range values', () => {
        it('custom classified colors - using custom labels and colors only', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", rangeClassification: LABELLED_RANGE_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: RANGE_CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'number'
                },
                classifyGeoJSONSync,
                ...DATASET_4
            });
            expect(data.length).toBe(2);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_4.data.map(d => d[DATASET_4.series[0].dataKey])).toEqual([0, 100, 500, 900]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 0, 900, 500, 100 ]);
            // original labels
            expect(DATASET_4.data.map(d => d[RANGE_CLASSIFICATIONS.dataKey])).toEqual([ 0, 100, 500, 900 ]);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['1) Between 0 and 100', '2) Between 100 and 1000', '2) Between 100 and 1000', '2) Between 100 and 1000']);
            expect(data[1].labels).toEqual(['Page A (1)', 'Page D (2)', 'Page C (2)', 'Page B (2)']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual([ '#ff0000', '#0000ff', '#0000ff', '#0000ff']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
        it('custom classified colors - using default labels and colors', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", rangeClassification: UNLABELLED_RANGE_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: RANGE_CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'number'
                },
                classifyGeoJSONSync,
                ...DATASET_4
            });
            expect(data.length).toBe(2);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_4.data.map(d => d[DATASET_4.series[0].dataKey])).toEqual([0, 100, 500, 900]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 0, 900, 500, 100 ]);
            // original labels
            expect(DATASET_4.data.map(d => d[RANGE_CLASSIFICATIONS.dataKey])).toEqual([ 0, 100, 500, 900 ]);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['1) >= 0<br>< 100', '2) >= 100<br><= 1000', '2) >= 100<br><= 1000', '2) >= 100<br><= 1000']);
            expect(data[1].labels).toEqual(['Page A (1)', 'Page D (2)', 'Page C (2)', 'Page B (2)']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual([ '#ff0000', '#0000ff', '#0000ff', '#0000ff']);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
        it('custom classified colors - using templatized labels and custom colors only - pie charts', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "", rangeClassification: PIE_CHART_TEMPLATE_LABELS_RANGE_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'pie',
                autoColorOptions,
                classifications: RANGE_CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'number'
                },
                classifyGeoJSONSync,
                ...DATASET_4
            });
            expect(data.length).toBe(1);
            expect(data[0].type).toBe('pie');
            expect(data[0].textposition).toEqual('inside');
            // original values
            expect(DATASET_4.data.map(d => d[DATASET_4.series[0].dataKey])).toEqual([0, 100, 500, 900]);
            // data values mapped and sorted
            expect(data[0].values).toEqual([ 0, 900, 500, 100 ]);
            // original labels
            expect(DATASET_4.data.map(d => d[RANGE_CLASSIFICATIONS.dataKey])).toEqual([ 0, 100, 500, 900 ]);
            // data labels mapped and sorted
            expect(data[0].labels).toEqual(['Page A - value: Between 0 and 100', 'Page D - value: Between 100 and 1000', 'Page C - value: Between 100 and 1000', 'Page B - value: Between 100 and 1000']);
            // colors are those defined by the user
            expect(data[0].marker.colors).toEqual([ '#ff0000', '#00ff00', '#00ff00', '#00ff00' ]);
            // LAYOUT
            expect(layout.margin).toEqual({t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true}); // fixed margins
        });
    });
    describe('Line/Bar chart common features', () => {
        function testAllTypes(props, handler) {
            ['line', 'bar'].map(type => toPlotly({
                type,
                ...props,
                classifyGeoJSONSync
            })).map(handler);
        }
        it('basic line/bar options', () => {
            testAllTypes(DATASET_1, ({data, layout}) => {
                // DATA
                expect(data.length).toBe(1);
                // expect(data[0].type).toBe('line');

                // data values mapped
                data[0].y.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.series[0].dataKey]));
                // data labels mapped
                data[0].x.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.xAxis.dataKey]));
                // LAYOUT

                // minimal margins, bottom automatic
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });

                // colors generated are the defaults, generated on series (1 color for series, so 1)
                // expect(layout.colorway).toEqual(defaultColorGenerator(1, COLOR_DEFAULTS));

                // yaxis
                expect(layout.yaxis.automargin).toBeTruthy();
                expect(layout.yaxis.showticklabels).toBeTruthy();
                expect(layout.yaxis.showgrid).toBeFalsy();

                // xaxis
                expect(layout.xaxis.automargin).toBeTruthy();
                expect(layout.xaxis.tickangle).toEqual('auto');
            });
        });
        it('yAxisLabel to be used as name of the chart', () => {
            testAllTypes({
                ...DATASET_1,
                yAxisLabel: "TEST_LABEL"
            }, ({ data }) => {
                expect(data[0].name).toEqual("TEST_LABEL");
            });
        });
        it('cartesian to show/hide grid', () => {
            testAllTypes({
                ...DATASET_1,
                cartesian: true
            }, ({ layout }) => {
                // bottom margin is optimized
                expect(layout.yaxis.showgrid).toBe(true);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });

        });
        it('nTicks passed to force to show all labels, max nTicks', () => {
            testAllTypes({
                ...DATASET_1,
                xAxisOpts: { nTicks: 200}
            }, ({ layout }) => {
                // bottom margin is optimized
                expect(layout.xaxis.nticks).toBe(200);
            });

        });

        it('check yAxis, prefix, format, suffix', () => {
            testAllTypes({
                ...DATASET_1,
                yAxisOpts: { tickPrefix: "test", format: ".2s", tickSuffix: "W/h" }
            }, ({ layout }) => {
                // bottom margin is optimized
                expect(layout.yaxis.tickprefix).toBe("test");
                expect(layout.yaxis.tickformat).toBe(".2s");
                expect(layout.yaxis.ticksuffix).toBe("W/h");
            });
        });
        it('check formula', () => {
            testAllTypes({
                ...DATASET_1,
                formula: "value * 2"
            }, ({ data }) => {
                // bottom margin is optimized
                data[0].y.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.series[0].dataKey] * 2));
            });
        });
    });
    describe('3) color coded/custom classified Bar chart with absolute values', () => {
        it('custom classified colors - using custom labels and colors only', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", classification: LABELLED_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(2);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_2.data[i][j][SPLIT_DATASET_2.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_2.data[i][j][SPLIT_DATASET_2.xAxis.dataKey]));
                // data labels mapped
                const classLabel = LABELLED_CLASSIFICATION.filter(item => item.value === SPLIT_DATASET_2.data[i][0][CLASSIFICATIONS.dataKey])[0]?.title;
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = LABELLED_CLASSIFICATION.filter(item => item.value === SPLIT_DATASET_2.data[i][0][CLASSIFICATIONS.dataKey])[0].color;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });
        it('custom classified colors - using default labels and colors', () => {
            const classification = UNLABELLED_CLASSIFICATION_3;
            const autoColorOptions = {
                defaultCustomColor: "#00ff00",
                defaultClassLabel: "Default",
                classification,
                name: 'global.colors.custom'
            };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_3
            });
            expect(data.length).toBe(3);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_3.data[i][j][SPLIT_DATASET_3.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_3.data[i][j][SPLIT_DATASET_3.xAxis.dataKey]));
                // data labels mapped
                const classLabel = classification.filter(item => item.value === SPLIT_DATASET_3.data[i][0][CLASSIFICATIONS.dataKey])[0]?.value ?? autoColorOptions.defaultClassLabel;
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = classification.filter(item => item.value === SPLIT_DATASET_3.data[i][0][CLASSIFICATIONS.dataKey])[0]?.color ?? autoColorOptions.defaultCustomColor;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });
        it('custom classified colors - using default labels and colors, wrong order', () => {
            const classification = UNLABELLED_CLASSIFICATION_5_ORDERED;
            const autoColorOptions = {
                defaultCustomColor: "#00ff00",
                defaultClassLabel: "Default",
                classification,
                name: 'global.colors.custom'
            };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classificationAttr: "classValue",
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_5_UNORDERED
            });
            expect(data.length).toBe(3);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_5_ORDERED.data[i][j][SPLIT_DATASET_5_ORDERED.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_5_ORDERED.data[i][j][SPLIT_DATASET_5_ORDERED.xAxis.dataKey], "here 2"));
                // data labels mapped
                const classLabel = classification.filter(item => item.value === SPLIT_DATASET_5_ORDERED.data[i][0][CLASSIFICATIONS.dataKey])[0]?.value ?? autoColorOptions.defaultClassLabel;
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = classification.filter(item => item.value === SPLIT_DATASET_5_ORDERED.data[i][0][CLASSIFICATIONS.dataKey])[0]?.color ?? autoColorOptions.defaultCustomColor;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });

        it('custom classified colors - using templatized labels and custom colors only - bar charts', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", classification: TEMPLATE_LABELS_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(2);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_2.data[i][j][SPLIT_DATASET_2.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_2.data[i][j][SPLIT_DATASET_2.xAxis.dataKey]));
                // data labels mapped
                const classLabel = TEMPLATE_LABELS_CLASSIFICATION
                    .filter(item => item.value === SPLIT_DATASET_2.data[i][0][CLASSIFICATIONS.dataKey])[0]?.title
                    .replace('${legendValue}', SPLIT_DATASET_2.series[0].dataKey);
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = TEMPLATE_LABELS_CLASSIFICATION.filter(item => item.value === SPLIT_DATASET_2.data[i][0][CLASSIFICATIONS.dataKey])[0].color;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });
        it('custom bar color', () => {
            const autoColorOptions = {
                defaultCustomColor: "#0888A1",
                defaultClassLabel: "Default",
                name: 'global.colors.custom',
                base: 190,
                range: 0,
                s: 0.95,
                v: 0.63
            };
            const { data } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(1);
            expect(data[0].type).toBe('bar');
            expect(data[0].marker.color).toBe(autoColorOptions.defaultCustomColor);
        });
    });
    describe('4) color coded/custom classified Bar chart with range values', () => {
        it('custom classified colors - using custom labels and colors only', () => {
            const autoColorOptions = {
                defaultCustomColor: "#00ff00",
                defaultClassLabel: "Default",
                rangeClassification: LABELLED_RANGE_CLASSIFICATION,
                name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: RANGE_CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'number'
                },
                classifyGeoJSONSync,
                ...DATASET_4
            });
            expect(data.length).toBe(2);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_4.data[i][j][SPLIT_DATASET_4.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_4.data[i][j][SPLIT_DATASET_4.xAxis.dataKey]));
                // data labels mapped
                const classLabel = LABELLED_RANGE_CLASSIFICATION.filter(
                    ({ min, max }) => trace.y[0] >= min && trace.y[0] < max
                )[0].title;
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = LABELLED_RANGE_CLASSIFICATION.filter(
                    ({min, max}) => trace.y[0] >= min && trace.y[0] < max
                )[0].color;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });
        it('custom classified colors - using default labels and colors', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", rangeClassification: UNLABELLED_RANGE_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: RANGE_CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'number'
                },
                classifyGeoJSONSync,
                ...DATASET_4
            });
            expect(data.length).toBe(2);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_4.data[i][j][SPLIT_DATASET_4.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_4.data[i][j][SPLIT_DATASET_4.xAxis.dataKey]));
                // data labels mapped
                const valuesInRange = trace.y.every(item =>
                    item >= UNLABELLED_RANGE_CLASSIFICATION[i].min && item < UNLABELLED_RANGE_CLASSIFICATION[i].max
                );
                expect(valuesInRange).toBe(true);
                const labelMinValue = UNLABELLED_RANGE_CLASSIFICATION[i].min;
                const labelMaxValue = UNLABELLED_RANGE_CLASSIFICATION[i].max;
                const classLabel = `>= ${labelMinValue}<br><${i === (traces.length - 1) ? '=' : ''} ${labelMaxValue}`;
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = UNLABELLED_RANGE_CLASSIFICATION[i].color;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });

        it('custom classified colors - using templatized labels and custom colors only - bar charts', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", rangeClassification: TEMPLATE_LABELS_RANGE_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: RANGE_CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'number'
                },
                classifyGeoJSONSync,
                ...DATASET_4
            });
            expect(data.length).toBe(2);
            const traces = data;
            traces.forEach((trace, i) => {
                expect(trace.type).toBe('bar');
                // data values mapped
                trace.y.map((v, j) => expect(v).toBe(SPLIT_DATASET_4.data[i][j][SPLIT_DATASET_4.series[0].dataKey]));
                trace.x.map((v, j) => expect(v).toBe(SPLIT_DATASET_4.data[i][j][SPLIT_DATASET_4.xAxis.dataKey]));
                const valuesInRange = trace.y.every(item =>
                    item >= TEMPLATE_LABELS_RANGE_CLASSIFICATION[i].min && item < TEMPLATE_LABELS_RANGE_CLASSIFICATION[i].max
                );
                expect(valuesInRange).toBe(true);
                // data labels mapped
                const labelValues = LABELLED_RANGE_CLASSIFICATION[i].title;
                const classAttributeLabel = RANGE_CLASSIFICATIONS.dataKey;
                const classLabel = `${classAttributeLabel} - ${labelValues}`;
                expect(trace.name).toBe(classLabel);
                // colors are those defined by the user
                const classColor = TEMPLATE_LABELS_RANGE_CLASSIFICATION[i].color;
                expect(trace.marker.color).toBe(classColor);
                expect(layout.margin).toEqual({ t: 8, b: 8, l: 8, r: 8, pad: 4, autoexpand: true });
            });
        });
    });
    describe('color coded/custom classified Bar chart - common features', () => {
        it('default classified bar chart type is group', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", classification: LABELLED_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(2);
            // we change in group because in some cases is not possible to use stack
            // in particular when using multiple traces
            expect(layout.barmode).toBe('group');
        });
        it('change classified bar chart type to grouped', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "Default", classification: LABELLED_CLASSIFICATION, name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                classifications: CLASSIFICATIONS,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2,
                barChartType: 'group'
            });
            expect(data.length).toBe(2);
            expect(layout.barmode).toBe('group');
        });
        it('expect data to be sorted in ascending order', () => {
            const { data } = toPlotly({
                xAxis: 'xAxis',
                classifyGeoJSONSync,
                ...DATASET_WITH_DATES
            });
            expect(DATASET_WITH_DATES.type).toBe('line');
            expect(data[0].x[0] <= data[0].x[1]).toBeTruthy();
        });
        it('color mapping classification is undefined - Bar Chart', () => {
            const autoColorOptions = { defaultCustomColor: "#00ff00", defaultClassLabel: "", name: 'global.colors.custom' };
            const { data, layout } = toPlotly({
                type: 'bar',
                autoColorOptions,
                options: {
                    classificationAttributeType: 'string'
                },
                classifyGeoJSONSync,
                ...DATASET_2
            });
            expect(data.length).toBe(1);
            // expect(data[0].type).toBe('line');

            // data values mapped
            data[0].y.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.series[0].dataKey]));
            // data labels mapped
            data[0].x.map((v, i) => expect(v).toBe(DATASET_1.data[i][DATASET_1.xAxis.dataKey]));
            // LAYOUT

            // colors generated are the defaults, generated on series (1 color for series, so 1)
            expect(data[0].marker.color).toBe(autoColorOptions.defaultCustomColor);

            // yaxis
            expect(layout.yaxis.automargin).toBeTruthy();
            expect(layout.yaxis.showticklabels).toBeTruthy();
            expect(layout.yaxis.showgrid).toBeFalsy();

            // xaxis
            expect(layout.xaxis.automargin).toBeTruthy();
            expect(layout.xaxis.tickangle).toEqual('auto');
        });
    });
});
