/**
 * Copyright 2016, GeoSolutions Sas.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

import expect from 'expect';
import { find, get } from 'lodash';
import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-dom/test-utils';

import MetadataModal from '../Save';

describe('This test for dashboard save form', () => {


    beforeEach((done) => {
        document.body.innerHTML = '<div id="container"></div>';
        setTimeout(done);
    });

    afterEach((done) => {
        ReactDOM.unmountComponentAtNode(document.getElementById("container"));
        document.body.innerHTML = '';
        setTimeout(done);
    });

    // test DEFAULTS
    it('creates the component with defaults, show=false', () => {
        const metadataModalItem = ReactDOM.render(<MetadataModal show={false}/>, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const metadataModalItemDom = ReactDOM.findDOMNode(metadataModalItem);
        expect(metadataModalItemDom).toNotExist();

        const getModals = function() {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };
        expect(getModals().length).toBe(0);

    });


    it('creates the component with a format error', () => {
        const metadataModalItem = ReactDOM.render(<MetadataModal show errors={["FORMAT"]} useModal id="MetadataModal"/>, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const getModals = function() {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };

        expect(getModals().length).toBe(1);

        const modalDivList = document.getElementsByClassName("modal-content");
        const closeBtnList = modalDivList.item(0).querySelectorAll('.modal-footer button');

        expect(closeBtnList.length).toBe(2);

        const errorFORMAT = modalDivList.item(0).querySelector('.errorFORMAT');
        expect(errorFORMAT).toExist();
    });

    it('creates the component with a size error', () => {
        const metadataModalItem = ReactDOM.render(<MetadataModal show errors={["SIZE"]} useModal id="MetadataModal"/>, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const getModals = function() {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };

        expect(getModals().length).toBe(1);

        const modalDivList = document.getElementsByClassName("modal-content");
        const closeBtnList = modalDivList.item(0).querySelectorAll('.modal-footer button');
        expect(closeBtnList.length).toBe(2);

        const errorSIZE = modalDivList.item(0).querySelector('.errorSIZE');
        expect(errorSIZE).toExist();
    });

    it('modal show permissions editor whe user is admin', () => {
        const user = {role: 'ADMIN'};
        const metadataModalItem = ReactDOM.render(<MetadataModal show user={user} useModal id="MetadataModal"/>, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const getModals = function() {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };

        expect(getModals().length).toBe(1);

        const permissionSection = document.querySelector(".permissions-table");
        expect(permissionSection).toExist();
    });

    it('modal show permissions editor with user is owner', () => {
        const user = {role: 'USER', name: 'geo'};
        const resource = {attributes: {owner: 'geo'}};
        const metadataModalItem = ReactDOM.render(<MetadataModal show user={user} resource={resource} useModal id="MetadataModal"/>, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const getModals = function() {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };

        expect(getModals().length).toBe(1);

        const permissionSection = document.querySelector(".permissions-table");
        expect(permissionSection).toExist();
    });

    it('modal hide permissions editor when user is neither admin nor owner', () => {
        const user = {role: 'USER', name: 'solution'};
        const resource = {attributes: {owner: 'geo'}};
        const metadataModalItem = ReactDOM.render(<MetadataModal show user={user} resource={resource} useModal id="MetadataModal"/>, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const getModals = function() {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };

        expect(getModals().length).toBe(1);

        const permissionSection = document.querySelector(".permissions-table");
        expect(permissionSection).toBeNull;
    });
    it('modal shows permissions when the map is new (no owner)', () => {
        const user = { role: 'USER', name: 'solution' };
        const resource = {  };
        const metadataModalItem = ReactDOM.render(<MetadataModal show user={user} resource={resource} useModal id="MetadataModal" />, document.getElementById("container"));
        expect(metadataModalItem).toExist();

        const getModals = () => {
            return document.getElementsByTagName("body")[0].getElementsByClassName('modal-dialog');
        };

        expect(getModals().length).toBe(1);

        const permissionSection = document.querySelector(".permissions-table");
        expect(permissionSection).toExist();
    });
    it('modal save button is disabled when enableFileDrop=true and fileDropStatus !== accepted', () => {
        const user = {role: 'ADMIN'};
        const metadataModalItem = ReactDOM.render(<MetadataModal user={user}
            show resource={{}} useModal enableFileDrop fileDropStatus="rejected" id="MetadataModal"/>, document.getElementById('container'));
        expect(metadataModalItem).toExist();

        const buttons = document.getElementsByTagName('button');
        const saveButton = find(buttons, button => button.childNodes[0] && (button.childNodes[0].textContent === 'save' ||
            get(button.childNodes[0].childNodes[0], 'textContent') === 'save'));
        expect(saveButton).toExist();
        expect(saveButton.classList.contains('disabled')).toBe(true);
    });
    it('modal save button is enabled when enableFileDrop=true and fileDropStatus === accepted', () => {
        const user = {role: 'ADMIN'};
        const metadataModalItem = ReactDOM.render(<MetadataModal user={user} show resource={{metadata: {name: 'res'}}} useModal
            enableFileDrop fileDropStatus="accepted" id="MetadataModal"/>, document.getElementById('container'));
        expect(metadataModalItem).toExist();

        const buttons = document.getElementsByTagName('button');
        const saveButton = find(buttons, button => button.childNodes[0] && (button.childNodes[0].textContent === 'save' ||
            get(button.childNodes[0].childNodes[0], 'textContent') === 'save'));
        expect(saveButton).toExist();
        expect(saveButton.classList.contains('disabled')).toBe(false);
    });
    it('modal save button is disabled when resource is loading', () => {
        const user = {role: 'ADMIN'};
        const Modal = ReactDOM.render(<MetadataModal user={user} show loading  id="map-save"/>, document.getElementById('container'));
        expect(Modal).toBeTruthy();

        const buttons = document.getElementsByTagName('button');
        const saveButton = find(buttons, button => button.childNodes[0] && (button.childNodes[0].textContent === 'save' ||
            get(button.childNodes[0].childNodes[0], 'textContent') === 'save'));
        const closeButton = find(buttons, button => button.childNodes[0] && (button.childNodes[0].textContent === 'close' ||
            get(button.childNodes[0].childNodes[0], 'textContent') === 'close'));
        const closeButtonModal = document.querySelector('.glyphicon-1-close');

        // Save button
        expect(saveButton).toBeTruthy();
        expect(saveButton.classList.contains('disabled')).toBe(true);
        // Close button
        expect(closeButton).toBeTruthy();
        expect(closeButton.classList.contains('disabled')).toBe(true);
        // Close button on modal header
        expect(closeButtonModal).toBeFalsy();
    });
    it('modal save button is disabled when save is in progress', () => {
        const user = {role: 'ADMIN'};
        const Modal = ReactDOM.render(<MetadataModal user={user} show  id="map-save"/>, document.getElementById('container'));
        expect(Modal).toBeTruthy();

        const buttons = document.getElementsByTagName('button');
        const saveButton = find(buttons, button => button.childNodes[0] && (button.childNodes[0].textContent === 'save' ||
            get(button.childNodes[0].childNodes[0], 'textContent') === 'save'));
        expect(saveButton).toBeTruthy();
        TestUtils.Simulate.click(saveButton);
        expect(saveButton.classList.contains('disabled')).toBe(true);
    });
    it('modal display spinner when loading', () => {
        const user = {role: 'ADMIN'};
        const Modal = ReactDOM.render(<MetadataModal user={user} loading show  id="map-save"/>, document.getElementById('container'));
        expect(Modal).toBeTruthy();
        const [spinner] = document.getElementsByClassName('mapstore-inline-loader');
        expect(spinner).toBeTruthy();
    });
    it('modal hide spinner when error', () => {
        const user = {role: 'ADMIN'};
        let Modal = ReactDOM.render(<MetadataModal  user={user} show  id="map-save"/>, document.getElementById('container'));
        expect(Modal).toBeTruthy();
        const buttons = document.getElementsByTagName('button');
        const saveButton = find(buttons, button => button.childNodes[0] && (button.childNodes[0].textContent === 'save' ||
            get(button.childNodes[0].childNodes[0], 'textContent') === 'save'));
        TestUtils.Simulate.click(saveButton);
        Modal = ReactDOM.render(<MetadataModal errors={["Network error"]}  user={user} show  id="map-save"/>, document.getElementById('container'));
        expect(Modal).toBeTruthy();
        const [spinner] = document.getElementsByClassName('mapstore-inline-loader');
        expect(spinner).toBeFalsy();
    });
});
