
/**
 * Copyright 2016, GeoSolutions Sas.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */

import PropTypes from 'prop-types';
import React from 'react';
import { Col, Grid, Label, Overlay, Popover, Row } from 'react-bootstrap';
import { findDOMNode } from 'react-dom';
import { Combobox, NumberPicker } from 'react-widgets';
import numberLocalizer from 'react-widgets/lib/localizers/simple-number';

import Message from '../I18N/Message';
import colorsSchema from './EqualIntervalComponents/ColorRamp';
import ColorRampItem from './EqualIntervalComponents/ColorRampItem';
import colors from './EqualIntervalComponents/ExtendColorBrewer';
import Button from '../misc/Button';

numberLocalizer();


class EqualInterval extends React.Component {
    static propTypes = {
        min: PropTypes.number,
        max: PropTypes.number,
        classes: PropTypes.number,
        onChange: PropTypes.func,
        onClassify: PropTypes.func,
        ramp: PropTypes.string,
        error: PropTypes.object
    };

    static contextTypes = {
        messages: PropTypes.object
    };

    static defaultProps = {
        min: 0,
        max: 100,
        classes: 5,
        ramp: "Blues",
        onChange: () => {},
        onClassify: () => {},
        error: null
    };

    shouldComponentUpdate() {
        return true;
    }

    getColorsSchema = () => {
        return this.props.classes ?
            colorsSchema.filter((c) => {
                return c.max >= this.props.classes;
            }, this) : colorsSchema;
    };

    getRampValue = () => {
        let ramp = this.props.ramp;
        if (!colors[this.props.ramp][this.props.classes]) {
            ramp = colorsSchema.filter((color) => { return color.max >= this.props.classes; }, this)[0].name;
        }
        return ramp;
    };

    renderErrorPopOver = () => {
        return (
            <Overlay
                target={() => findDOMNode(this.refs[this.props.error.type])}
                show placement="top" >
                <Popover>
                    <Label bsStyle="danger" > <Message msgId={this.props.error.msg}/></Label>
                </Popover>
            </Overlay>
        );
    };

    render() {
        return (
            <Grid fluid>
                <Row>
                    <Col xs={4}>
                        <Row><Col xs={12} >
                            <label><Message msgId="equalinterval.min"/></label>
                        </Col></Row>
                        <Row><Col xs={12} >
                            <NumberPicker
                                ref="min"
                                onChange={this.changeMin}
                                value={this.props.min}
                                format="-#,###.##"
                                precision={3}
                            />
                            {this.props.error && this.props.error.type === 'min' ? this.renderErrorPopOver() : null}
                        </Col></Row>
                    </Col>
                    <Col xs={4}>
                        <Row><Col xs={12} >
                            <label><Message msgId="equalinterval.max"/></label>
                        </Col></Row>
                        <Row><Col xs={12} >
                            <NumberPicker
                                ref="max"
                                onChange={this.changeMax}
                                value={this.props.max}
                                format="-#,###.##"
                                precision={3}
                            />
                            {this.props.error && this.props.error.type === 'max' ? this.renderErrorPopOver() : null}
                        </Col></Row>
                    </Col>
                    <Col xs={4}>
                        <Row>
                            <Col xs={12} >
                                <label><Message msgId="equalinterval.classes"/></label>
                            </Col>
                        </Row>
                        <Row>
                            <Col xs={12} >
                                <NumberPicker
                                    onChange={(number) => this.props.onChange("classes", number)}
                                    precision={0} min={3} max={12} step={1}
                                    value={this.props.classes}
                                />
                            </Col>
                        </Row>
                    </Col>
                </Row>
                <Row>
                    <Col xs={6} >
                        <Row><Col xs={12} >
                            <label><Message msgId="equalinterval.ramp"/></label>
                        </Col></Row>
                        <Row><Col xs={12} >
                            <Combobox data={this.getColorsSchema()}
                                groupBy="schema"
                                onChange={(value) => this.props.onChange("ramp", value.name)}
                                textField="name"
                                itemComponent={ColorRampItem} value={this.getRampValue()}/>
                        </Col></Row>
                    </Col>
                    <Col xs={6}>
                        <Row style={{paddingTop: "25px"}}>
                            <Col xs={12} >
                                <Button disabled={this.classifyDisabled()}
                                    onClick={this.generateEqualIntervalRamp}
                                    style={{"float": "right"}}>
                                    <Message msgId="equalinterval.classify"/></Button>
                            </Col></Row>
                    </Col>
                </Row>
            </Grid>);
    }

    classifyDisabled = () => {
        return this.props.error && this.props.error.type ? true : false;
    };

    generateEqualIntervalRamp = () => {
        let ramp = colors[this.getRampValue()][this.props.classes];
        let min = this.props.min;
        let max = this.props.max;
        let step = (max - min) / this.props.classes;
        let colorRamp = ramp.map((color, idx) => {
            return {color: color, quantity: min + idx * step};
        });
        this.props.onClassify("colorRamp", colorRamp);
    };

    changeMin = (value) => {
        if (value < this.props.max) {
            if (this.props.error) {
                this.props.onChange("error", {});
            }
            this.props.onChange("min", value);
        } else {
            this.props.onChange("error", {
                type: "min",
                msg: "equalinterval.minerror"
            });
        }

    };

    changeMax = (value) => {
        if (value > this.props.min) {
            if (this.props.error) {
                this.props.onChange("error", {});
            }
            this.props.onChange("max", value);
        } else {
            this.props.onChange("error", {
                type: "max",
                msg: "equalinterval.maxerror"
            });
        }
    };
}

export default EqualInterval;
